Option Strict On

Imports System.Messaging
Imports Sales
Imports System.EnterpriseServices
Imports System.Reflection

' Supply the COM+ application name. 
<Assembly: ApplicationName("SalesOrders")> 
' Supply a COM+ Type.
<Assembly: ApplicationActivation(ActivationOption.Library)> 
' Define where the strong name Key pair is
<Assembly: AssemblyKeyFileAttribute("KeyPair.snk")> 

' Example: Transactional Queue that has no error
Public Class Example5a
    Shared Sub Main()
        Dim objMessageQueue As MessageQueue
        Dim objTrans As MessageQueueTransaction
        Try
            objMessageQueue = New MessageQueue(Constants.MQ.LocalTransQueue)
            ' Create a transaction object to be used on all sends
            objTrans = New MessageQueueTransaction()

            ' Create an instance of the simple Order Class and populate
            ' it with a few products
            Dim objOrder As New Order()
            objOrder.CustomerID = "ALFKI"
            With objOrder.OrderDetails.Add()
                .ProductID = 1
                .Quantity = 99
                .UnitPrice = 17.5
                .Discount = 0
            End With
            With objOrder.OrderDetails.Add()
                .ProductID = 2
                .Quantity = 20
                .UnitPrice = 18
                .Discount = 0
            End With

            Dim objMessage As New Message()
            With objMessage
                .Body = objOrder
                .Label = "(Example5a)XML - Order:" & objOrder.CustomerID
                .Priority = MessagePriority.Highest
                .Formatter = New XmlMessageFormatter()
            End With
            ' Send message within a transaction
            objTrans.Begin()
            objMessageQueue.Send(objMessage, objTrans)
            objTrans.Commit()
        Catch ex As Exception
            ' An error occured so we need to rollback any trans if we started one
            If objTrans.Status = MessageQueueTransactionStatus.Pending Then
                objTrans.Abort()
            End If
            Debug.WriteLine(ex.Message)
            Debug.WriteLine(ex.StackTrace)
        Finally
            objMessageQueue.Close()
        End Try
    End Sub
End Class

' Example: Transactional Queue that has an error
Public Class Example5b
    Shared Sub Main()
        ' Example 2: Sending a full object as a message
        Dim objMessageQueue As MessageQueue
        Dim objTrans As MessageQueueTransaction
        Try
            objMessageQueue = New MessageQueue(Constants.MQ.LocalTransQueue)
            ' Create a transaction object to be used on all sends
            objTrans = New MessageQueueTransaction()

            ' Create an instance of the simple Order Class and populate
            ' it with a few products
            Dim objOrder As New Order()
            objOrder.CustomerID = "ALFKI"
            With objOrder.OrderDetails.Add()
                .ProductID = 1
                .Quantity = 99
                .UnitPrice = 17.5
                .Discount = 0
            End With
            With objOrder.OrderDetails.Add()
                .ProductID = 2
                .Quantity = 20
                .UnitPrice = 18
                .Discount = 0
            End With

            Dim objMessage As New Message()
            With objMessage
                .Body = objOrder
                .Label = "(Example5b)XML - Order:" & objOrder.CustomerID
                .Priority = MessagePriority.Highest
                '.Formatter = New BinaryMessageFormatter(Runtime.Serialization.Formatters.FormatterAssemblyStyle.Full, Runtime.Serialization.Formatters.FormatterTypeStyle.TypesWhenNeeded)
                .Formatter = New XmlMessageFormatter()
            End With
            ' Send message within a transaction
            objTrans.Begin()
            objMessageQueue.Send(objMessage, objTrans)
            ' Cause an error here, within a transaction
            objMessageQueue.Path = "xxxx"
            objMessageQueue.Send(objMessage, objTrans)
            objTrans.Commit()
        Catch ex As Exception
            ' An error occured so we need to rollback any trans if we started one
            If objTrans.Status = MessageQueueTransactionStatus.Pending Then
                objTrans.Abort()
            End If
            Debug.WriteLine(ex.Message)
            Debug.WriteLine(ex.StackTrace)
        Finally
            objMessageQueue.Close()
        End Try
    End Sub
End Class

' Enter the world of COM+ !
<Transaction(TransactionOption.RequiresNew)> Public Class Example5c
    Inherits ServicedComponent

    Shared Sub SendMessage(ByVal blnError As Boolean)
        Dim objMessageQueue As MessageQueue
        Dim objTrans As MessageQueueTransaction
        Try
            objMessageQueue = New MessageQueue(Constants.MQ.LocalTransQueue)
            ' Create a transaction object to be used on all sends
            objTrans = New MessageQueueTransaction()

            ' Create an instance of the simple Order Class and populate
            ' it with a few products
            Dim objOrder As New Order()
            objOrder.CustomerID = "ALFKI"
            With objOrder.OrderDetails.Add()
                .ProductID = 1
                .Quantity = 99
                .UnitPrice = 17.5
                .Discount = 0
            End With
            With objOrder.OrderDetails.Add()
                .ProductID = 2
                .Quantity = 20
                .UnitPrice = 18
                .Discount = 0
            End With

            Dim objMessage As New Message()
            With objMessage
                .Body = objOrder
                .Label = "(Example5c)XML - Order:" & objOrder.CustomerID
                .Priority = MessagePriority.Highest
                .Formatter = New XmlMessageFormatter()
                ' We want to know if it was fully received
                .AcknowledgeType = AcknowledgeTypes.NegativeReceive
                .AdministrationQueue = New MessageQueue(Constants.MQ.LocalAckQueue)
            End With
            ' Send message within a transaction
            objTrans.Begin()
            objMessageQueue.Send(objMessage, objTrans)
            ' We will change the CustomerID to one that is not valid
            ' this will generate an error when the message is processed
            If blnError Then
                objOrder.CustomerID = "AAAAA"
            Else
                objOrder.CustomerID = "ANATR"
            End If
            objMessage.Label = "(Example5c)XML - Order:" & objOrder.CustomerID
            objMessageQueue.Send(objMessage, objTrans)
            objTrans.Commit()
        Catch ex As Exception
            ' An error occured so we need to rollback any trans if we started one
            If objTrans.Status = MessageQueueTransactionStatus.Pending Then
                objTrans.Abort()
            End If
            Debug.WriteLine(ex.Message)
            Debug.WriteLine(ex.StackTrace)
        Finally
            objMessageQueue.Close()
        End Try
    End Sub

    Sub ProcessMessage()
        Dim objMessageQueue As MessageQueue
        Try
            ' Retreiving an object from a message queue
            objMessageQueue = New MessageQueue(Constants.MQ.LocalTransQueue)
            objMessageQueue.Formatter = New XmlMessageFormatter(New Type() {GetType(Order)})

            Debug.WriteLine(ContextUtil.TransactionId)
            Do While objMessageQueue.GetAllMessages.Length <> 0
                Dim objMessage As Message = objMessageQueue.Receive(New TimeSpan(0, 0, 10), MessageQueueTransactionType.Automatic)
                Dim objOrder As Order = DirectCast(objMessage.Body, Order)
                ' Process message
                objOrder.Save()
                MessageBox.Show("Processed Order for Customer: " & objOrder.CustomerID)
            Loop
            ContextUtil.SetComplete()
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error")
            ContextUtil.SetAbort()
        Finally
            objMessageQueue.Close()
        End Try
    End Sub
End Class